/*

The BinaryTree Class

@author Autumn C. Spaulding <a href="mailto:autumn@max.cs.kzoo.edu">email</a>
Creation Date: 24 July 2000

Modifications:
    Modifier: Alyce Brady
    Modification Date: November 11, 2002
    Modifications Made: Modifications to documentation (e.g., to remove
        empty preconditions); added levelOrderTraversal;
        also modified to use NodeAction interface.
       
    Modifier: Pamela Cutter
    Modification Date; July 2003
    Modifications Made:  Modified to define a binary tree by using a 
    	reference to a TreeNode as the root.

Modifications:
    Modifier: studentName
    Modification Date: currentDate
    Modifications Made:

Description:
    This file contains some of the implementation of a BinaryTree class. 
    It is intended as an outline and starting point for the "Binary Trees"
    lab in the Data Structures course.  The implementation is based on a 
    recursive definition of a tree.
    
    A binary tree is a reference to a TreeNode, called the root.  A TreeNode
    contains data and two children, left and right, which are themselves 
    TreeNodes.
    
    In this implementation, an empty tree is represented by a null TreeNode.
    A leaf node is represented by a node with a data value and two null 
    references.  The leftTree and rightTree methods allow us to make subtrees 
    from the left and right children of the root node.  By using these 
    subtrees, we can take advantage of the recursive nature of a tree.  To 
    create these subtrees, an additional constructor was added to create a
    Binary Tree with a specific node as its root.
*/

public class BinaryTree
{
    //data:
    protected TreeNode root;
    
        
    /*tested*/
    /** Creates an empty binary tree with no data and no children. */
    public BinaryTree()
    {
        root = null;
    }
    
    /*tested*/
    /** Creates a binary tree with the given TreeNode object as its root. */
    public BinaryTree(TreeNode initNode)
    {
    	root = initNode;
    }
    
    /*tested*/
    /** Tests whether this is an empty tree.
            @return true if the tree is empty; false otherwise
    */
    public boolean isEmpty()
    {
        return root == null;
    }
   
    /*tested*/
    /** Gets the data associated with the root node of this particular tree
        (recall recursive definition of trees).
            @return value associated with tree's root node; 
                          null if tree is empty
    */
    public Object getElement()
    {
        return root.getValue();
    }

    /*tested*/
    /** Gets the left child of the current tree.
            @return the left child as a tree
    */
    public BinaryTree leftTree()
    {
        return new BinaryTree(root.getLeft());
    }

    /*tested*/
    /** Gets the right child of the current tree.
            @return the right child as a tree
    */
    public BinaryTree rightTree()
    {
        return new BinaryTree(root.getRight());
    }

    /** Adds elements to a binary tree.  This implementation adds the
        elements in breadth-first (top-down, left-to-right) order.
            @param value the value to be added to the tree.
            @return true when the value has been added; should never return false
    */
    public boolean add(Object value)
    {
        Queue queue = new LLQueue();
        queue.enqueue(this.root);
        while( ! queue.isEmpty() )
        {
            TreeNode node = (TreeNode)queue.dequeue();
			
            //if the current position is null, then we know we can place a
            //value here.
            //place the value in that position in the tree, and create new 
            //BinaryTrees for its children, which will both initially be null.
            if (node == null) 	// empty tree
            {
            	root = new TreeNode(value);
            	return true;
            }            
            else 	// (node ! = null)
            {
            	if (node.getLeft() == null)
            	{
            		node.setLeft(new TreeNode(value));
                	return true;
            	}
            	else if (node.getRight() == null)
            	{
            		node.setRight(new TreeNode(value));
            		return true;
            	}
            	else 
            	{
		            //otherwise, if the position is not null (that is, we can't place
		            //it at the current position), then we add the left and right children
		            //to the queue (if we can) and go back to the beginning of the loop.
		            queue.enqueue(node.getLeft());
		            queue.enqueue(node.getRight());
		        }
		    }
        }
        return false;    //this statement should never be executed.
    }

	/** Traverses the tree in breadth-first order.
	        @param action an object that will act on all the nodes in the tree
	*/
	public void breadthFirstTraversal(NodeVisitor action)
	{
		Queue queue = new LLQueue();
		queue.enqueue(this.root);
		while( ! queue.isEmpty() )
		{
			TreeNode node = (TreeNode)queue.dequeue();
			if ( node != null )
            {
    			action.visit(node.getValue());
    			queue.enqueue(node.getLeft());
    			queue.enqueue(node.getRight());
            }
		}
	}	
	
	    
 }    //end class BinaryTree
