/*

The BST Class

@author Alyce Brady
Creation Date: 12 November 2002
Modifications:  June 2003, by Pam Cutter, added methods to allow deletion of elements
					in the tree, as well as for equals.
				July 2003, by Pam Cutter.  Modified to use the TreeNode class

*/

public class BST extends BinaryTree
{

    public BST()
    {
    	super();
    }
    
    public BST(TreeNode initNode)
    {
    	super(initNode);
    }
    
    /** Adds elements to a binary search tree.
        (Precondition: <code>value</code> is <code>Comparable</code>.)
            @param value the value to be added to the tree.
            @return true when the value has been added; should never return false
    */
    public boolean add(Object value)
    {
    	Comparable val = (Comparable) value;
    	if ( isEmpty() )
			root = new TreeNode(val);
        else if ( val.compareTo((Comparable)getElement()) < 0 )
        {
           	if (leftTree().isEmpty())
           		root.setLeft(new TreeNode(val));
           	else
           	{
           		BST BSTLeftTree = new BST(root.getLeft());
           		BSTLeftTree.add(val);
           	}
        }
        else
        {
        	if (rightTree().isEmpty())
        		root.setRight(new TreeNode(val));
        	else
        	{
        		BST BSTRightTree = new BST(root.getRight());
           		BSTRightTree.add(val);
            }
        }
        return true;
           
    }
    
    
    /** Returns the leftMost value in the tree.
    
    */
    private Object leftmost()
    {
    	if (leftTree().isEmpty())
    		return getElement();
    	else return ((BST)leftTree()).leftmost();
    
    	
    }
    
    /** Removes the leftmost value in the tree, and returns this value
    */
    private Object removeLeftmost()
    {
    	if (leftTree().isEmpty())
    	{
    		Object temp = getElement();
    		if (!rightTree().isEmpty())
    		{
    			root.setValue(rightTree().getElement());
    			root.setLeft(root.getRight().getLeft());
    			root.setRight(root.getRight().getRight());
    		}
    		else 
    			root.setValue(null);  
    		return temp;
    	}
    	else
    	{
    		BST BSTLeftTree = new BST(root.getLeft());
    		return BSTLeftTree.removeLeftmost();
    	}
    
    }
    
    /** Removes elements from the BST  
    	@param obj the value to be removed from the tree.
        @return true when the obj has been removed; false otherwise
    */
	public boolean remove(Comparable obj)
	{
		if (isEmpty())
			return false;
		
		int comp = obj.compareTo(getElement());
		
		if (comp == 0)			// we have found the object
		{
			if (rightTree().isEmpty())
				if (leftTree().isEmpty())
					root.setValue(null);
				else
				{
					root.setValue(leftTree().getElement());
					root.setRight(root.getLeft().getRight());
					root.setLeft(root.getLeft().getLeft());
				}
			else
			{
				BST BSTRightTree = new BST(root.getRight());
				root.setValue(BSTRightTree.removeLeftmost());
			}
			return true;
		}
		else if (comp < 0)				// obj should be in left subtree
		{
			BST BSTLeftTree = new BST(root.getLeft());
			return BSTLeftTree.remove(obj);
		}
		else
		{
			BST BSTRightTree = new BST(root.getRight()); 
			return BSTRightTree.remove(obj);	// obj should be in right subtree
		}
	}    
	
	
	/** Determines whether two BSTs are equal, uses the super.equals method
		@param o is an Object that should be a BST
		@return true if the two BSTs are equal; false otherwise
	 */
	public boolean equals(Object o)
	{
		if (! (o instanceof BST))
			return false;
			
		BST otherTree = (BST) o;
		return super.equals(otherTree);
	
	}

}    //end class BST
