/*
	Main file for Binary Tree Lab
	@author Autumn C. Spaulding
	Creation Date: 24 July 2000
	
	More thorough documentation may be found within the BinaryTree class file.
*/

//import java.lang.Comparable;

public class BSTLab
{

	public static void main(String args[])
	{
		// Debug.turnOn();
		Debug.turnOff();


		//construct an empty binary tree here.
		BST tree = new BST();
		
		
		//insert elements in level order here.
		tree.add(new Integer(12));
		tree.add(new Integer(7));
		tree.add(new Integer(3));
		tree.add(new Integer(4));
		tree.add(new Integer(8));
		tree.add(new Integer(25));
		tree.add(new Integer(0));
		tree.add(new Integer(142));
		tree.add(new Integer(17));
		tree.add(new Integer(26));


		//traverse the tree in breadth-first order to see what you have done.
        NodeVisitor printer = new PrintAction();
		System.out.println("******Traversing Tree: breadth-first order******");
		tree.breadthFirstTraversal(printer);
		ExtremeValueCalculator extremes = new ExtremeValueCalculator(); 
		tree.breadthFirstTraversal(extremes);
		System.out.println("Min value = " + extremes.getMinimum() + "; max value = "
		    + extremes.getMaximum());
		System.out.println("The tree is " + (isBST(tree) ? " " : " not ") + " a BST.");
		
		System.out.println("Tree has "+tree.numNodes()+" nodes");
		System.out.println("Tree has " +tree.numLeaves()+" leaves");
		System.out.println("Tree has "+tree.numOccurrences(new Integer(5))+" occurrences of 5");
		tree.add(new Integer(5));
		System.out.println("Tree has "+tree.numOccurrences(new Integer(5))+" occurrences of 5");
		System.out.println("Tree contains 142: "+tree.contains(new Integer(142)));
		System.out.println("Tree contains 11: "+tree.contains(new Integer(11)));
		
		System.out.println("****** Traversing pre-order **********");
		tree.preOrderTraversal(printer);
		System.out.println("****** Traversing in-order **********");
		tree.inOrderTraversal(printer);
		System.out.println("****** Traversing post-order **********");
		tree.postOrderTraversal(printer);
		
		/*
		// construct a bst
		BST bst = new BST();
		bst.add(new Integer(12));
		bst.add(new Integer(7));
		bst.add(new Integer(3));
		bst.add(new Integer(4));
		bst.add(new Integer(8));
		bst.add(new Integer(25));
		bst.add(new Integer(0));
		bst.add(new Integer(142));
		bst.add(new Integer(17));
		bst.add(new Integer(26));

		//traverse the tree in breadth-first order to see what you have done.
        printer = new PrintAction();
		System.out.println("******Traversing Tree: breadth-first order******");
		bst.breadthFirstTraversal(printer);
	    extremes = new ExtremeValueCalculator(); 
		bst.breadthFirstTraversal(extremes);
		System.out.println("Min value = " + extremes.getMinimum() + "; max value = "
		    + extremes.getMaximum());
		System.out.println("The tree is" + (isBST(bst) ? " " : " not ") + "a BST.");
		System.out.println("The trees are" +(tree.equals(bst) ? " " : " not ") + "equal.");
		*/	
		// test the remove method of BST
		tree.remove(new Integer(11));
		System.out.println("traversal after remove 11");
		tree.breadthFirstTraversal(printer);
		
		
		//test the equals method of BST
		BST bst2 = new BST();
		bst2.add(new Integer(12));
		bst2.add(new Integer(10));
		
		System.out.println("The trees are" +(tree.equals(bst2) ? " " : " not ") + "equal.");
	
	}

    public static boolean isBST(BinaryTree tree)
    {
        if ( tree.isEmpty() )
            return true;
        Comparable value = (Comparable) tree.getElement();
		ExtremeValueCalculator extremes = new ExtremeValueCalculator(); 
		tree.leftTree().breadthFirstTraversal(extremes);
        Comparable maxLeft = extremes.getMaximum();
        if ( maxLeft != null && value.compareTo(maxLeft) <= 0 )
            return false;
		tree.rightTree().breadthFirstTraversal(extremes);
        Comparable minRight = extremes.getMinimum();
        if ( minRight != null && value.compareTo(minRight) < 0 )
            return false;
        return ( isBST(tree.leftTree()) && isBST(tree.rightTree()) );
    }

}	//end class BinaryTreeLab
