/*
    Implementation of ExtremeValueCalculator class.
    @author Alyce Brady
    Creation Date: Fall 2002
    
    The ExtremeValueCalculator class implements the NodeVisitor interface.
	It keeps track of the smallest and largest values it sees, assuming
	that the values are Comparable.
*/


public class ExtremeValueCalculator implements NodeVisitor
{
    private Comparable smallest;
    private Comparable largest;

    /** Constructs an ExtremeValueCalculator object. **/
    public ExtremeValueCalculator()
	{
	    smallest = largest = null;
	}

    /** Compares data value to smallest and largest values seen
     *  so far; remembers this data value if it is the smallest
     *  or largest.
     *  (Precondition: <code>data</code> is <code>Comparable</code>.)
     **/
    public void visit(Object data)
	{
	    if ( data == null )
	        return;

        Comparable compData = (Comparable) data;
        if ( smallest == null || compData.compareTo(smallest) < 0 )
            smallest = compData;
        if ( largest == null || compData.compareTo(largest) > 0 )
            largest = compData;
	}

    /** Returns the smallest value seen (visited).
     *      @return minimum value visited, or null if no non-null values
     *              were visited
     **/
    public Comparable getMinimum()
	{
	    return smallest;
	}

    /** Returns the largest value seen (visited).
     *      @return maximum value visited, or null if no non-null values
     *              were visited
     **/
    public Comparable getMaximum()
	{
	    return largest;
	}

}
