// Class: LLQueue
//
// Author: Alyce Brady
//
// License Information:
//   This class is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation.
//
//   This class is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

import java.util.LinkedList;

/**
 *  
 *  <code>LLQueue</code> class implements the Queue interface using
 *  a <code>LinkedList</code> as its internal data structure.
 *
 *  @author Alyce Brady
 *  @version 1 November 2002
 *  @see Visitor
 **/
public class LLQueue implements Queue
{
    private LinkedList theList;

    /** Constructs an object that implements the Queue interface. **/
    public LLQueue()
    {
        theList = new LinkedList();
    }

    /** Returns <code>true</code> if this queue is empty;
     *  <code>false</code> otherwise. */
    public boolean isEmpty()
    {
        return theList.isEmpty();
    }

    /** Adds a specified object to the "back" of this queue.
     *    @param x the object to add to the queue
     */
    public void enqueue(Object x)
    {
        theList.addLast(x);
    }

    /** Removes the element at the "front" of this queue.
     *    @returns the returned element
     *    @throws NoSuchElementException if the queue is empty
     **/
    public Object dequeue()
    {
        return theList.removeFirst();
    }

    /** Returns the element at the "front" of this queue, without
     *  modifying the queue.
     *    @throws NoSuchElementException if the queue is empty
     **/
    public Object peekFront()
    {
        return theList.getFirst();
    }

}
